@@ -7,19 +7,20 @@ import botocore
 import click
 import json
 
-from newrelic_lambda_cli.api import NewRelicGQL
-from newrelic_lambda_cli.cliutils import failure, success
+from newrelic_lambda_cli.cliutils import failure, success, warning
 from newrelic_lambda_cli.functions import get_function
 from newrelic_lambda_cli.types import (
     IntegrationInstall,
     IntegrationUninstall,
     IntegrationUpdate,
 )
+from newrelic_lambda_cli.utils import catch_boto_errors, NR_DOCS_ACT_LINKING_URL
 
 INGEST_STACK_NAME = "NewRelicLogIngestion"
 LICENSE_KEY_STACK_NAME = "NewRelicLicenseKeySecret"
 
 __cached_license_key_policy_arn = None
+__cached_license_key_nr_account_id = None
 
 
 def _get_role(session, role_name):
@@ -44,10 +45,27 @@ def _check_for_ingest_stack(session):
     return _get_cf_stack_status(session, INGEST_STACK_NAME)
 
 
-def _get_cf_stack_status(session, stack_name):
+def _get_cf_stack_status(session, stack_name, nr_account_id=None):
     """Returns the status of the CloudFormation stack if it exists"""
     try:
         res = session.client("cloudformation").describe_stacks(StackName=stack_name)
+        if nr_account_id is not None:
+            stack_output_account_id = _get_stack_output_value(session, ["NrAccountId"])
+            # Checking length of outputs here to protect against installs done with older CLI versions.
+            # We don't want to constantly warn users who installed on previous versions with no outputs.
+            if (
+                len(stack_output_account_id) > 0
+                and str(nr_account_id) not in stack_output_account_id
+            ):
+                warning(
+                    "WARNING: Managed secret already exists in this region for New Relic account {0}.\n"
+                    "Current CLI behavior limits the setup of one managed secret per region.\n"
+                    "To set up an additional secret for New Relic account {1} see our docs:\n{2}.\n"
+                    "Or run this command with --disable-license-key-secret to avoid attemping to create a new managed secret.".format(
+                        stack_output_account_id, nr_account_id, NR_DOCS_ACT_LINKING_URL
+                    )
+                )
+
     except botocore.exceptions.ClientError as e:
         if (
             e.response
@@ -276,6 +294,7 @@ def _exec_change_set(client, change_set, mode, stack_name=INGEST_STACK_NAME):
         success("Done")
 
 
+@catch_boto_errors
 def update_log_ingestion_function(input):
     assert isinstance(input, IntegrationUpdate)
 
@@ -386,6 +405,7 @@ def update_log_ingestion_function(input):
     )
 
 
+@catch_boto_errors
 def remove_log_ingestion_function(input):
     assert isinstance(input, IntegrationUninstall)
 
@@ -406,6 +426,7 @@ def remove_log_ingestion_function(input):
     success("Done")
 
 
+@catch_boto_errors
 def create_integration_role(input):
     """
     Creates a AWS CloudFormation stack that adds the New Relic AWSLambda Integration
@@ -445,6 +466,7 @@ def create_integration_role(input):
     )
 
 
+@catch_boto_errors
 def remove_integration_role(input):
     """
     Removes the AWS CloudFormation stack that includes the New Relic AWS Integration
@@ -470,26 +492,7 @@ def remove_integration_role(input):
         success("Done")
 
 
-def validate_linked_account(gql, input):
-    """
-    Ensure that the aws account associated with the 'provider account',
-    if it exists, is the same as the aws account of the default aws-cli
-    profile configured in the local machine.
-    """
-    assert isinstance(gql, NewRelicGQL)
-    assert isinstance(input, IntegrationInstall)
-
-    account = gql.get_linked_account_by_name(input.linked_account_name)
-    if account is not None:
-        aws_account_id = get_aws_account_id(input.session)
-        if aws_account_id != account["externalId"]:
-            raise click.UsageError(
-                "The selected linked AWS account [%s] does not match "
-                "the AWS account of your AWS profile [%s]."
-                % (account["externalId"], aws_account_id)
-            )
-
-
+@catch_boto_errors
 def install_log_ingestion(
     input,
     nr_license_key,
@@ -532,6 +535,7 @@ def install_log_ingestion(
     return True
 
 
+@catch_boto_errors
 def update_log_ingestion(input):
     """
     Updates the New Relic AWS Lambda log ingestion function and role.
@@ -569,6 +573,7 @@ def update_log_ingestion(input):
         return True
 
 
+@catch_boto_errors
 def get_log_ingestion_license_key(session):
     """
     Fetches the license key value from the log ingestion function
@@ -579,6 +584,7 @@ def get_log_ingestion_license_key(session):
     return None
 
 
+@catch_boto_errors
 def auto_install_license_key(input):
     """
     If the LK secret is missing, create it, picking up the LK value from the ingest
@@ -599,66 +605,78 @@ def auto_install_license_key(input):
     return True
 
 
+@catch_boto_errors
 def install_license_key(input, nr_license_key, policy_name=None, mode="CREATE"):
     assert isinstance(input, (IntegrationInstall, IntegrationUpdate))
-    lk_stack_status = _get_cf_stack_status(input.session, LICENSE_KEY_STACK_NAME)
-    if lk_stack_status is None:
-        click.echo(
-            "Setting up %s stack in region: %s"
-            % (LICENSE_KEY_STACK_NAME, input.session.region_name)
-        )
-        try:
-            client = input.session.client("cloudformation")
+    lk_stack_status = _get_cf_stack_status(
+        input.session, LICENSE_KEY_STACK_NAME, input.nr_account_id
+    )
+    if lk_stack_status is not None:
+        success("Managed secret already exists")
+        return True
 
-            update_mode = mode == "UPDATE"
-            parameters = []
-            if policy_name is not None:
-                parameters.append(
-                    {"ParameterKey": "PolicyName", "ParameterValue": policy_name}
-                )
-            elif update_mode:
-                parameters.append(
-                    {"ParameterKey": "PolicyName", "UsePreviousValue": True}
-                )
+    click.echo(
+        "Setting up %s stack in region: %s"
+        % (LICENSE_KEY_STACK_NAME, input.session.region_name)
+    )
+    try:
+        client = input.session.client("cloudformation")
 
+        update_mode = mode == "UPDATE"
+        parameters = []
+        if policy_name is not None:
             parameters.append(
-                {"ParameterKey": "LicenseKey", "ParameterValue": nr_license_key}
+                {"ParameterKey": "PolicyName", "ParameterValue": policy_name}
+            )
+        elif update_mode:
+            parameters.append({"ParameterKey": "PolicyName", "UsePreviousValue": True})
+
+        parameters.extend(
+            (
+                {"ParameterKey": "LicenseKey", "ParameterValue": nr_license_key},
+                {
+                    "ParameterKey": "NrAccountId",
+                    "ParameterValue": str(input.nr_account_id),
+                },
+            )
+        )
+
+        change_set_name = "%s-%s-%d" % (
+            LICENSE_KEY_STACK_NAME,
+            mode,
+            int(time.time()),
+        )
+        click.echo("Creating change set: %s" % change_set_name)
+        template_path = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)),
+            "templates",
+            "license-key-secret.yaml",
+        )
+
+        with open(template_path) as template:
+            change_set = client.create_change_set(
+                StackName=LICENSE_KEY_STACK_NAME,
+                TemplateBody=template.read(),
+                Parameters=parameters,
+                Capabilities=["CAPABILITY_NAMED_IAM"],
+                Tags=[{"Key": key, "Value": value} for key, value in input.tags]
+                if input.tags
+                else [],
+                ChangeSetType=mode,
+                ChangeSetName=change_set_name,
             )
 
-            change_set_name = "%s-%s-%d" % (
-                LICENSE_KEY_STACK_NAME,
-                mode,
-                int(time.time()),
+            _exec_change_set(
+                client, change_set, mode, stack_name=LICENSE_KEY_STACK_NAME
             )
-            click.echo("Creating change set: %s" % change_set_name)
-            template_path = os.path.join(
-                os.path.dirname(os.path.abspath(__file__)),
-                "templates",
-                "license-key-secret.yaml",
-            )
-
-            with open(template_path) as template:
-                change_set = client.create_change_set(
-                    StackName=LICENSE_KEY_STACK_NAME,
-                    TemplateBody=template.read(),
-                    Parameters=parameters,
-                    Capabilities=["CAPABILITY_NAMED_IAM"],
-                    Tags=[{"Key": key, "Value": value} for key, value in input.tags]
-                    if input.tags
-                    else [],
-                    ChangeSetType=mode,
-                    ChangeSetName=change_set_name,
-                )
-
-                _exec_change_set(
-                    client, change_set, mode, stack_name=LICENSE_KEY_STACK_NAME
-                )
-        except Exception as e:
-            failure("Failed to create %s stack: %s" % (LICENSE_KEY_STACK_NAME, e))
-            return False
-    return True
+    except Exception as e:
+        failure("Failed to create %s stack: %s" % (LICENSE_KEY_STACK_NAME, e))
+        return False
+    else:
+        return True
 
 
+@catch_boto_errors
 def remove_license_key(input):
     assert isinstance(input, (IntegrationUninstall, IntegrationUpdate))
     client = input.session.client("cloudformation")
@@ -685,11 +703,24 @@ def remove_license_key(input):
         success("Done")
 
 
-def _get_license_key_policy_arn(session):
-    """Returns the policy ARN for the license key secret if it exists"""
+def _get_license_key_outputs(session):
+    """Returns the account id and policy ARN for the license key secret if they exist"""
+    global __cached_license_key_nr_account_id
     global __cached_license_key_policy_arn
-    if __cached_license_key_policy_arn:
-        return __cached_license_key_policy_arn
+    if __cached_license_key_nr_account_id and __cached_license_key_policy_arn:
+        return [__cached_license_key_nr_account_id, __cached_license_key_policy_arn]
+
+    account_id, policy_arn = _get_stack_output_value(
+        session, ["NrAccountId", "ViewPolicyARN"]
+    )
+    if account_id is not None:
+        __cached_license_key_nr_account_id = account_id
+    if policy_arn is not None:
+        __cached_license_key_policy_arn = policy_arn
+    return [account_id, policy_arn]
+
+
+def _get_stack_output_value(session, output_keys):
     client = session.client("cloudformation")
     try:
         stacks = client.describe_stacks(StackName=LICENSE_KEY_STACK_NAME).get(
@@ -708,14 +739,15 @@ def _get_license_key_policy_arn(session):
         if not stacks:
             return None
         stack = stacks[0]
-        output_key = "ViewPolicyARN"
-        for output in stack.get("Outputs", []):
-            if output["OutputKey"] == output_key:
-                __cached_license_key_policy_arn = output["OutputValue"]
-                return __cached_license_key_policy_arn
-        else:
-            return None
+        output_values = []
+        for output_key in output_keys:
+            for output in stack.get("Outputs", []):
+                if output["OutputKey"] == output_key:
+                    value = output["OutputValue"] or None
+                    output_values.append(value)
+        return output_values
 
 
+@catch_boto_errors
 def get_aws_account_id(session):
     return session.client("sts").get_caller_identity().get("Account")
